# -*- coding:utf-8 -*-
# @time:2022/12/2316:33
# @author:LX
# @file:test_barset.py
# @software:PyCharm

'''

    进度条集合测试
'''

from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    sys,
    QApplication,
    QMainWindow,
    QThread,
    Signal,
    QColor,
    QSlider
)

from PyQtGuiLib.core.progressBar import WaterBar
from PyQtGuiLib.core.progressBar import CircularBar
from PyQtGuiLib.core.progressBar import LoadBar

class DurationTimeThread(QThread):
    added = Signal(int)
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)


    def run(self) -> None:
        n=1
        while True:
            self.added.emit(n)
            n+=1
            self.msleep(100)
            if n == 100+1:
                print("完成")
                break

class TestPullOverWidget(QMainWindow):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(800,500)

        self.setObjectName("test")
#         self.setStyleSheet('''
# #test{
# background-color: rgb(0, 0, 0);
# }
#         ''')

        self.waterbar = WaterBar(self)
        self.waterbar.move(100,250)

        # 加载进度条
        self.loadbar = LoadBar(self)
        self.loadbar.move(250,100)
        self.loadbar.resize(350,45)


        # 圆环进度条
        self.cir = CircularBar(self)
        self.cir.resize(120, 120)
        self.cir.setVariableLineSegment(CircularBar.Double)
        self.cir.setOuterStyle(CircularBar.CustomDashLine)
        self.cir.setOuterDashPattern([2, 3, 5, 6])
        self.cir.setInnerStyle(CircularBar.DashLine)
        self.cir.setTextSize(15)
        self.cir.move(50, 50)

        self.th = DurationTimeThread()
        self.th.added.connect(self.test)
        self.th.start()


    def test(self, n):
        self.cir.setValue(n)
        self.loadbar.setValue(n)
        self.waterbar.setValue(n)



if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = TestPullOverWidget()
    win.show()

    if PYQT_VERSIONS in ["PyQt6", "PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())




'''
    测试 控件的组成分析函数
'''


from PyQtGuiLib.core.resolver import dumpStructure

import sys
from PyQt5.QtWidgets import QApplication,QWidget,QPushButton,QTabWidget,QScrollArea,QCheckBox


app = QApplication(sys.argv)
dumpStructure(QTabWidget())




# -*- coding:utf-8 -*-
# @time:2022/12/2710:59
# @author:LX
# @file:test_gradientBar.py
# @software:PyCharm
from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    sys,
    QApplication,
    QWidget,
    QColor,
    QResizeEvent
)

from PyQtGuiLib.core.widgets import (
    RoundWidget,   # 圆角QWidget窗口
    BorderlessMainWindow, # 无边框矩形主窗口
    BorderlessFrame,      # 无边框矩形QFrame窗口
    BorderlessWidget,     # 无边框矩形QWidget窗口
    BorderlessStackedWidget  # 无边框矩形StackedWidget窗口
    )

from PyQtGuiLib.core.widgets import TitleBar
'''
    测试 标题栏
'''

class TestTitleBar(BorderlessWidget):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(800,500)

        self.setStyleSheet('''
#widget{
background-color:#d4d4d4;
border-radius:30px;
}
        ''')
        self.tbar = TitleBar(self)
        self.tbar.setTitlePos(TitleBar.Title_Left)
        self.tbar.setTitleText("测试标题栏")
        self.tbar.setTitleColor(QColor(100,200,78))
        # self.tbar.setTitleIcon(r"/Applications/Python 3.8/save/PyQtGuiLib/PyQtGuiLib/tests/image/1.png")
        self.tbar.setBtnStyle(TitleBar.MacStyle)
        self.tbar.setStyleSheet("background-color: rgb(232, 232, 232);")

    def resizeEvent(self, event:QResizeEvent) -> None:
        super().resizeEvent(event)
        self.tbar.updateTitleSize()

if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = TestTitleBar()
    win.show()

    if PYQT_VERSIONS in ["PyQt6","PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


import importlib

# 当前python qt使用的版本
PYQT_VERSIONS = None

VERSIONS = ["PyQt5","PySide2","PySide6","PyQt6"]

for module in VERSIONS:
    try:
        importlib.import_module(module)
        PYQT_VERSIONS = module
        break
    except:
        pass

if PYQT_VERSIONS is None:
    raise ModuleNotFoundError("Module not found,support:PyQt5,PyQt6,PySide2,PySide6")



from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    sys,
    QApplication,
    QWidget,
    QThread,
    Signal,
    QPoint,
    QFont,
    QColor,
    QPen,
    QPainter,
    QPaintEvent,
    QFontMetricsF,
    QSize,
    QResizeEvent,
    QMainWindow
)



class TestPullOverWidget(QMainWindow):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(500,500)


    def paintEvent(self, event:QPaintEvent) -> None:
        painter = QPainter()
        painter.begin(self)

        for i in range(self.width()):
            for j in range(self.height()):
                painter.drawPoint(QPoint(i,j))

        # painter.setBrush(QColor(105, 255, 152))
        # painter.drawRect(10,30,100,100)
        #
        # painter.drawArc(150,30,100,100,0,360*16)
        # painter.setOpacity(0.5)
        # painter.setPen(QColor(105, 255, 152))
        # painter.drawEllipse(150,30,100,100)
        painter.end()



if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = TestPullOverWidget()
    win.show()

    if PYQT_VERSIONS == "PyQt6":
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())



from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    QApplication,
    QWidget,
    QFrame,
    QPainter,
    QPaintEvent,
    QRect,
    QPen,
    QFont,
    QFontMetricsF,
    QColor,
    QPushButton,
    QPropertyAnimation,
    QParallelAnimationGroup,
    QBrush,
    qt,
    QSize,
    QPoint,
    QPixmap,
    QIcon,
    QStyle,
    QStyleOption,
    desktopCenter,
    desktopSize,
    textSize
)

'''
    窗口的标题栏
'''


# 缩小,放大,关闭 三按钮基类
class ButtonIcon(QPushButton):

    '''
        缩小,放大,关闭
        win 风格
        mac 风格
    '''
    WinStyle = "WinStyle"
    MacStyle = "MacStyle"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.w,self.h = 20,20
        self.resize(self.w,self.h)
        
        self.btnStyle = ButtonIcon.MacStyle

        self.open_width = 1
        self.radius = 3

    def resize(self, *args) -> None:
        if len(args) == 1 and isinstance(args[0],QSize):
            self.w = args[0].width(),args[0].height()
        else:
            self.w,self.h = args[0],args[1]

        super().resize(self.w,self.h)

    # 设置 缩小,放大,关闭 的风格
    def setBtnStyle(self,style:str = "WinStyle"):
        self.btnStyle = style
    
    def isWinStyle(self) -> bool:
        return True if self.btnStyle == ButtonIcon.WinStyle else False

    def isMacStyle(self) -> bool:
        return True if self.btnStyle == ButtonIcon.MacStyle else False

    # 绘制图标,所有子类都必须实现都方法
    def drawIcon(self, painter: QPainter):
        pass

    def paintEvent(self, event: QPaintEvent) -> None:
        painter = QPainter(self)
        painter.setRenderHints(qt.Antialiasing | qt.SmoothPixmapTransform | qt.TextAntialiasing)

        self.drawIcon(painter)

        painter.end()


# 缩小按钮
class ZoomButton(ButtonIcon):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)

    def drawIcon(self,painter:QPainter):
        op = QPen()
        op.setWidth(self.open_width)
        painter.setPen(op)

        if self.isWinStyle():
            rect = QRect(1,1,self.width()-self.open_width-1,self.height()-self.open_width-1)
            painter.drawRoundedRect(rect,self.radius,self.radius)
            painter.drawLine(5,self.height()//2,self.width()-5,self.height()//2)
        elif self.isMacStyle():
            op.setColor(QColor(255, 199, 124))
            painter.setPen(op)
            brush = QBrush(QColor(255, 199, 124))
            painter.setBrush(brush)

            rect = QRect(1, 1, self.width() - self.open_width - 1, self.height() - self.open_width - 1)
            painter.drawRoundedRect(rect, self.w//2,self.h//2)
            op.setColor(QColor(50, 100, 100))
            painter.setPen(op)
            painter.drawLine(5, self.height() // 2, self.width() - 5, self.height() // 2)
            painter.setBrush(qt.NoBrush)


# 放大按钮
class LargeButton(ButtonIcon):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)


    def drawIcon(self, painter: QPainter):
        op = QPen()
        op.setWidth(self.open_width)
        painter.setPen(op)

        if self.isWinStyle():
            rect1 = QRect(1, 1, 13,13)
            rect2 = QRect(6, 6, 13,13)
            painter.drawRoundedRect(rect1, self.radius,self.radius)
            painter.drawRoundedRect(rect2, self.radius,self.radius)
        elif self.isMacStyle():
            op.setColor(QColor(40, 194, 50))
            painter.setPen(op)
            brush = QBrush(QColor(40, 194, 50))
            painter.setBrush(brush)

            rect = QRect(1, 1, self.width() - self.open_width - 1, self.height() - self.open_width - 1)
            painter.drawRoundedRect(rect, self.w//2,self.h//2)

            brush = QBrush(QColor(0,0,0))
            painter.setBrush(brush)
            op.setColor(QColor(50, 100, 100))
            painter.setPen(op)
            painter.drawEllipse(self.width()//2-5,self.height()//2-5,5,5)
            painter.drawEllipse(self.width()//2+1,self.height()//2+1,5,5)

            painter.setBrush(qt.NoBrush)


# 关闭按钮
class CloseButton(ButtonIcon):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)

    def drawIcon(self, painter: QPainter):
        op = QPen()
        op.setWidth(self.open_width)
        painter.setPen(op)

        if self.isWinStyle():
            rect = QRect(1,1,self.width()-self.open_width-1,self.height()-self.open_width-1)
            painter.drawRoundedRect(rect,self.radius,self.radius)
            painter.drawLine(5,5,self.width()-5,self.height()-5)
            painter.drawLine(self.width()-5,5,5,self.height()-5)
        elif self.isMacStyle():
            op.setColor(QColor(252, 70, 70))
            painter.setPen(op)
            brush = QBrush(QColor(252, 70, 70))
            painter.setBrush(brush)

            rect = QRect(1, 1, self.width() - self.open_width - 1, self.height() - self.open_width - 1)
            painter.drawRoundedRect(rect, self.w//2,self.h//2)

            op.setColor(QColor(50, 100, 100))
            painter.setPen(op)

            painter.drawLine(5, 5, self.width() - 5, self.height() - 5)
            painter.drawLine(self.width() - 5, 5, 5, self.height() - 5)


class TitleBar(QWidget):

    # 标题的位置
    Title_Left = "TextLeft"
    Title_Center = "TitleCenter"

    '''
        缩小,放大,关闭
        win 风格
        mac 风格
    '''
    WinStyle = "WinStyle"
    MacStyle = "MacStyle"

    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)

        # 样式表生效
        self.setAttribute(qt.WA_StyledBackground,True)

        self.__parent = None #  type:QWidget

        if args:
            self.__parent = args[0]

        # 标题位置
        self.title_pos = TitleBar.Title_Left

        # 标题高度
        self.h = 30
        # 标题文本
        self.title_text= "Title"
        self.title_color = QColor(0,0,0)
        self.title_size = 15

        # 图标大小,图标路径
        self.image_size = 30,30
        self.title_icon_path = r""
        self.is_sync_icon = True # 是否同步桌面任务栏的图标

        # 保存窗口在放大之前的位置大小,已经窗口大小状态
        self.old_screen_geometry = QRect(0,0,0,0) # type:QRect
        self.screen_state = False

        # 动画时长
        self.ani_msec = 300  # 毫秒

        if self.__parent is not None:
            self.move(0,0)
            self.resize(self.__parent.width(),self.h)

        # 创建缩小,放大,关闭 按钮
        self.zm = ZoomButton(self)
        self.lm = LargeButton(self)
        self.cm = CloseButton(self)
        # 创建缩小,放大,关闭事件
        self.createEvent()

        # 设置标题设置
        self.setTitlePos(TitleBar.Title_Center)
        self.updateTitleSize()  # 更新标题栏位置

    def setTitleText(self,text:str):
        self.title_text = text

    def setTitleColor(self,color:QColor):
        self.title_color = color

    def setTitleSize(self,size:int):
        self.title_size = size

    def setAllTitle(self,text:str,color:QColor,size:int):
        self.setTitleText(text)
        self.setTitleColor(color)
        self.setTitleSize(size)

    # 设置标题的位置
    def setTitlePos(self,mode:str="TitleCenter"):
        self.title_pos = mode

    # 设置 缩小,放大,关闭 按钮的风格
    def setBtnStyle(self, style: str = "WinStyle"):
        self.zm.setBtnStyle(style)
        self.lm.setBtnStyle(style)
        self.cm.setBtnStyle(style)

    # 设置标题栏高度
    def setTitleHeight(self,h:int):
        self.h = h

    def setParent(self, parent:QWidget) -> None:
        self.__parent = parent
        super().setParent(parent)

        if self.__parent is not None:
            self.move(0,0)
            self.resize(self.__parent.width(),self.h)

    # 绘制标题
    def drawTitleText(self,painter: QPainter):
        # 绘制文字
        f = QFont()
        f.setPointSize(self.title_size)
        painter.setFont(f)

        op = QPen()
        op.setColor(self.title_color)
        painter.setPen(op)

        icon_distance = 0

        if self.title_icon_path:
            icon_distance = 30

        # 文字
        fs = textSize(f,self.title_text)
        fw = fs.width()
        fh = fs.height()
        if self.title_pos == TitleBar.Title_Center:
            painter.drawText(self.width() // 2 - fw // 2, self.height() // 2 + fh // 2, self.title_text)

        if self.title_pos == TitleBar.Title_Left:
            if PYQT_VERSIONS in ["PyQt6","PySide6"]: # 这两个版本的文字高度计算是精准的需要-5个像素
                painter.drawText(10+icon_distance, self.height() // 2 + fh // 2-5, self.title_text)
            else:
                painter.drawText(10+icon_distance, self.height() // 2 + fh // 2, self.title_text)

    # 绘制tub
    def drawTitleIcon(self,painter: QPainter):
        f = QFont()
        f.setPointSize(self.title_size)
        fs = textSize(f,self.title_text)
        fw = fs.width()

        pix = QPixmap(self.title_icon_path)
        if self.title_pos == TitleBar.Title_Center:
            painter.drawPixmap(self.width() // 2 - fw // 2-self.image_size[0], self.height() // 2 - self.image_size[1] // 2, *self.image_size,
                               pix)
        if self.title_pos == TitleBar.Title_Left:
            painter.drawPixmap(5, self.height()//2-self.image_size[1]//2, *self.image_size, pix)
        if self.is_sync_icon:
            self.__parent.setWindowIcon(QIcon(self.title_icon_path))

    def paintEvent(self, event: QPaintEvent) -> None:
        painter = QPainter(self)

        painter.setRenderHints(qt.Antialiasing | qt.SmoothPixmapTransform | qt.TextAntialiasing)

        self.drawTitleIcon(painter)
        self.drawTitleText(painter)

        # 自动调用
        self.updateTitleSize()

        painter.end()

    # 创建缩小,放大,关闭事件
    def createEvent(self):
        self.zm.clicked.connect(lambda :self.ani("zoom"))
        self.lm.clicked.connect(lambda :self.ani("arge"))
        self.cm.clicked.connect(lambda :self.ani("close"))

    # 设置动画的时长
    def setAniDuration(self,msec:int):
        self.ani_msec = msec

    # 设置图标
    def setTitleIcon(self,path:str,isWindowIcon:bool=True):
        '''

        :param path: 图片路径
        :param isWindowIcon: 是否同步桌面任务栏的图标
        :return:
        '''
        self.title_icon_path = path
        self.is_sync_icon = isWindowIcon

    # 设置是否同步桌面任务栏的图标
    def setSyncWindowIcon(self,b:bool):
        self.is_sync_icon = b

    # 缩小,放大,关闭 动画
    def ani(self,action:str):
        animation = QPropertyAnimation(self.__parent)
        animation.setTargetObject(self.__parent)
        animation.setDuration(self.ani_msec)

        if action == "zoom":
            def _t(parent,size):
                parent.showMinimized()
                parent.setGeometry(size)

            old_geometry = self.__parent.geometry()
            animation.setPropertyName(b"size")
            animation.setStartValue(self.__parent.size())
            animation.setEndValue(QSize(10,10))
            # animation.start()
            animation.finished.connect(lambda :_t(self.__parent,old_geometry))

            # 获取窗口位置的中心点
            center_point = QPoint(self.__parent.x()+self.__parent.width()//2,
                                  self.__parent.y()+self.__parent.height()//2)

            # 移动动画
            animation_move = QPropertyAnimation(self.__parent)
            animation_move.setPropertyName(b"pos")
            animation_move.setTargetObject(self.__parent)
            animation_move.setDuration(self.ani_msec)

            animation_move.setStartValue(self.pos())
            animation_move.setEndValue(center_point)

            # 动画组
            ani_group = QParallelAnimationGroup(self.__parent)
            ani_group.addAnimation(animation_move)
            ani_group.addAnimation(animation)
            ani_group.start()
        elif action == "arge":
            animation.setPropertyName(b"size")

            # 移动动画
            animation_move = QPropertyAnimation(self.__parent)
            animation_move.setPropertyName(b"pos")
            animation_move.setTargetObject(self.__parent)
            animation_move.setDuration(self.ani_msec)

            animation.setStartValue(self.__parent.size())
            animation_move.setStartValue(self.__parent.pos())

            if self.screen_state is False:
                # 保存旧属性和状态
                self.old_screen_geometry = self.__parent.geometry()  # type:QRect

                # 获取的单个屏幕的大小
                animation.setEndValue(desktopSize())
                #
                animation_move.setEndValue(QPoint(0,0))

                self.screen_state = True
            else:
                animation.setEndValue(self.old_screen_geometry.size())
                #
                animation_move.setEndValue(QPoint(self.old_screen_geometry.x(),self.old_screen_geometry.y()))
                self.screen_state = False
            # 动画组
            ani_group = QParallelAnimationGroup(self.__parent)
            ani_group.addAnimation(animation_move)
            ani_group.addAnimation(animation)
            ani_group.start()
        elif action == "close":
            animation.setPropertyName(b"windowOpacity")
            animation.setStartValue(1)
            animation.setEndValue(0)
            animation.start()
            animation.finished.connect(self.__parent.close)

    # 更新标题栏大小
    def updateTitleSize(self) -> None:
        self.move(0, 0)
        self.resize(self.__parent.width(), self.h)

        # 自动计算 缩小,放大,关闭 的位置
        btn_w_interval = 15 # 按钮直接的间隔
        # 按钮占据的总宽度
        btn_occupied_width = self.zm.width()+self.lm.width()+self.cm.width()+btn_w_interval*3
        occ_w = self.width()-btn_occupied_width
        zm_h = self.height()//2 - self.zm.height()//2
        lm_h = self.height()//2 - self.lm.height()//2
        cm_h = self.height()//2 - self.cm.height()//2
        self.zm.move(occ_w,zm_h)
        occ_w = occ_w+self.zm.width()+btn_w_interval
        self.lm.move(occ_w,lm_h)
        occ_w  = occ_w+self.lm.width()+btn_w_interval
        self.cm.move(occ_w,cm_h)



# -*- coding:utf-8 -*-
# @time:2022/12/1711:33
# @author:LX
# @file:__init__.py.py
# @software:PyCharm

from PyQtGuiLib.core.widgets.borderlessWidget import (
    BorderlessMainWindow,
    BorderlessFrame,
    BorderlessWidget,
    BorderlessStackedWidget
)
from PyQtGuiLib.core.widgets.roundWidget import RoundWidget
from PyQtGuiLib.core.widgets.buttonWidget import ButtonWidget
from PyQtGuiLib.core.widgets.titleBar import TitleBar
from PyQtGuiLib.core.widgets.statusBar import StatusBar



# -*- coding:utf-8 -*-
# @time:2022/12/2214:52
# @author:LX
# @file:loadBar.py
# @software:PyCharm
from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    sys,
    QApplication,
    QWidget,
    Signal,
    QFont,
    QColor,
    QPen,
    QPainter,
    QPaintEvent,
    QFontMetricsF,
    QSize,
    QResizeEvent,
    qt,
    textSize
)


'''
    加载进度条
'''
class LoadBar(QWidget):
    # 进度改变时,发出信号
    valueChange = Signal(int)

    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.w,self.h =30, 50

        self.edge_distance_x = 5  # 水平方向的边距
        self.outer_radius,self.inner_radius = 20, 15 # 内外圆角

        # 每一段大小
        self.degree = (self.w-self.edge_distance_x*4)/100
        # 当前值
        self.cu_value = 0
        self.max_value = 100

        # 外框和进度条的颜色
        self.outline_border_color = QColor(0,0,0)
        self.bar_color = QColor(0,82,70)

        # 边的宽度
        self.border_width = 3

        # 文字属性
        self.text = "100%"
        self.text_size = 15
        self.text_color = QColor(170, 85, 127)
        self.__is_hide_text = False  # 是否隐藏文字选项

        self.resize(300,45)

    def resize(self, *args) -> None:
        if len(args) == 1 and isinstance(args[0],QSize):
            self.w = args[0].width(),args[0].height()
        else:
            self.w,self.h = args[0],args[1]

        self.degree = (self.w-self.edge_distance_x*4)/self.max_value
        super().resize(self.w,self.h)

    def setText(self,text:str):
        self.text = text

    def setTextColor(self,color:QColor):
        self.text_color = color

    def setTextSize(self,size:int):
        self.text_size = size

    def setAllText(self,text:str,color:QColor,size:int):
        self.setText(text)
        self.setTextColor(color)
        self.setTextSize(size)

    def isHideText(self,b:bool):
        self.__is_hide_text = b

    def setValue(self,v:int):
        self.cu_value = self.degree * v
        self.update()
        # 进度文字
        self.text = "{}%".format(v)
        # 发送信号
        self.valueChange.emit(v)

    def value(self) -> int:
        return self.cu_value // self.degree

    # 设置进度条的外圆角大小
    def setOuterRadius(self,r:int):
        self.outer_radius = r

    # 设置进度条的内圆角大小
    def setInnerRadius(self,r:int):
        self.inner_radius = r

    # 设置进度条的内外圆角
    def setRadius(self,outer_r:int,inner_r:int):
        self.setOuterRadius(outer_r)
        self.setInnerRadius(inner_r)

    # 设置进度条边的宽
    def setBorderWidth(self,w:int):
        self.border_width = w

    # 进度条
    def drawBar(self,painter:QPainter):
        painter.setPen(QPen(self.outline_border_color,self.border_width))
        painter.drawRoundedRect(self.edge_distance_x,2,self.w-(self.edge_distance_x<<1),self.h-4,
                                self.outer_radius,self.outer_radius)

        painter.setBrush(self.bar_color)
        painter.setPen(QPen(self.bar_color, 1))

        painter.drawRoundedRect(self.edge_distance_x+5, 2+6, self.cu_value, self.h - 4-12,
                             self.inner_radius,self.inner_radius)

    # 绘制文字
    def drawText_(self,painter:QPainter):
        if not self.__is_hide_text:
            # 绘制文字
            f = QFont()
            f.setBold(True)
            f.setPointSize(self.text_size)
            painter.setFont(f)
            painter.setPen(self.text_color)
            # 文字
            fs = textSize(f,self.text)
            fw = fs.width()
            fh = fs.height()
            painter.drawText(self.w // 2 - fw // 2,self.h // 2 + fh // 4, self.text)

    def paintEvent(self, event:QPaintEvent) -> None:
        painter = QPainter()
        painter.begin(self)
        painter.setRenderHints(qt.Antialiasing | qt.SmoothPixmapTransform | qt.TextAntialiasing)

        # 绘制进度条
        self.drawBar(painter)

        # 绘制文字
        self.drawText_(painter)

        painter.end()

    def resizeEvent(self, e:QResizeEvent) -> None:
        self.w = e.size().width()
        self.h = e.size().height()
        super().resizeEvent(e)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = LoadBar()
    win.show()

    if PYQT_VERSIONS == "PyQt6":
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


from PyQtGuiLib.core.progressBar.circularBar import CircularBar
from PyQtGuiLib.core.progressBar.loadBar import LoadBar
from PyQtGuiLib.core.progressBar.waterBar import WaterBar
from PyQtGuiLib.core.progressBar.gradientBar import GradientBar


from PyQtGuiLib.header import (
    sys,
    QApplication,
    QWidget,
    QHBoxLayout,
    Qt,
    QPainter,
    QPaintEvent,
    QRect,
    QColor,
    QSize,
    QPen,
    QPainterPath
)



'''
    气泡窗口弃案
'''
class BubbleWidget(QWidget):
    def __init__(self,*args,**kwargs):
        super(BubbleWidget, self).__init__(*args,**kwargs)


        self.km = 0 # 默认三角形开口大小是宽度度六分之一
        self.triangle_h = 20 # 三角形高度
        self.w,self.h = 0,0 # 气泡大小
        self.base_edge_color = QColor(232, 232, 232)  # 底边颜色
        self.border_color = QColor(102, 204, 255) # 气泡边框颜色
        self.text = "Bubble"  # 默认文字
        self.resize(200,80)

        self.setObjectName("BubbleW")
        self.setAttribute(Qt.WA_TintedBackground)
        self.setWindowFlags(Qt.WindowShadeButtonHint|Qt.FramelessWindowHint)

        self.hlay = QHBoxLayout(self)
        self.hlay.setContentsMargins(1,1,1,1)
        self.setStyleSheet('''
/*#bubble{
border:1px solid gray;
border-radius:5px;
background-color:rgb(192, 245, 255);
}*/
        ''')

        self.bubble = QWidget()
        self.bubble.setObjectName("bubble")
        self.hlay.addWidget(self.bubble)

    def resize(self, w:int,h:int) -> None:
        self.w,self.h=w,h
        self.km = w//6
        super(BubbleWidget, self).resize(w,h)

    def setBorderColor(self,bcolor:QColor):
        self.border_color = bcolor

    def setText(self,text:str):
        self.text = text

    # 设置开口大小
    def setKm(self,km:int):
        self.km = km

    # 三角形
    def delta(self,painter:QPainter,x:int,km):
        '''

        :param painter: 对象
        :param x: 位置
        :param km: 开口大小
        :return:
        '''
        # 反锯齿渲染
        painter.setRenderHint(QPainter.Antialiasing)
        painter.setPen(QPen(self.border_color, 1, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))
        painter.drawLine(x,self.triangle_h,x+km,0)
        painter.drawLine(x+km,self.triangle_h,x+km,0)
        painter.setPen(self.base_edge_color)
        painter.drawLine(x+1,self.triangle_h,x+km-1,self.triangle_h)


    def paintEvent(self, e: QPaintEvent) -> None:
        painter = QPainter()
        painter.begin(self)
        painter.setPen(QPen(self.border_color, 1, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))
        painter.drawRoundedRect(QRect(1,self.triangle_h,self.w-2,self.h-self.triangle_h-2),5,5)
        painter.drawText(self.w//2-20,self.h//2+15,self.text)
        self.delta(painter,self.w//2,self.km)
        painter.end()
        # super(BubbleWidget, self).paintEvent(e)

class Test(QWidget):
    def __init__(self,*args,**kwargs):
        # super().__init__()
        super(Test, self).__init__(*args,**kwargs)
        self.resize(500,500)

        self.bu = BubbleWidget(self)
        self.bu.setText("蔚蓝色")
        self.bu.setKm(30)  # 气泡开口大小
        self.bu.move(100,50)

        self.bu1 = BubbleWidget(self)
        self.bu1.setKm(60)
        self.bu1.resize(100,80)
        self.bu1.setBorderColor(QColor(89, 90, 255))
        self.bu1.move(100,150)

        self.bu2 = BubbleWidget(self)
        self.bu2.resize(100,100)
        self.bu2.setKm(-40)
        self.bu2.setText("少女粉")
        self.bu2.setBorderColor(QColor(255, 69, 135))
        self.bu2.move(100,250)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = Test()
    win.show()
    sys.exit(app.exec_())


# -*- coding:utf-8 -*-
# @time:2023/1/87:12
# @author:LX
# @file:builtStyleDesigner.py
# @software:PyCharm
from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    sys,
    json,
    QApplication,
    QVBoxLayout,
    QResizeEvent,
    QTreeWidgetItem,
    QWidget,
    QSize,
    QPushButton,
    QColorDialog
)
from functools import partial
from PyQtGuiLib.styles.builtStyleDesigner.builtStyleDesignerUI import BuiltStyleDesignerUI
from PyQtGuiLib.styles import ButtonStyle
'''

    内置-样式设计器
'''

# 加载配置文件
def loadConfig()->dict:
    with open("controlConfig.json", "r") as f:
        data = json.load(f)
    return data


class BuiltStyleDesigner(BuiltStyleDesignerUI):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.resize(1100, 850)

        # 从配置文件中读取
        self.createTree(loadConfig())

        # 判断是否已经创建的控件,控件列表,控件数量
        self.is_Controls = False
        self.controls = []
        self.control_number = 35

        # 当前点击的控件名称,方法,当前点击的控件对象
        self.cu_click_name = ""
        self.cu_fun = ""
        self.cu_click_control_obj = None

        # 控制qss区域事件多次触发
        self.is_control_edit = False

        # ----------
        # 设置一个当前的控件数量
        self.c_number_spin.setValue(self.control_number)
        self.c_number_spin.lineEdit().setReadOnly(True) # 只禁用编辑功能

        # 事件
        self.myEvent()

    # 当前点击的控件对象
    def getClickObj(self)->QWidget:
        return self.cu_click_control_obj

    # 添加控件
    def addControls(self,widget:QWidget,size:QSize=QSize(130,60)):
        widget.setFixedSize(size)
        self.flow.addWidget(widget)

    def createTree(self,tree_dict:dict):
        if not tree_dict:
            return None

        tree = self.treeObj()

        for name,values in tree_dict.items():
            item = QTreeWidgetItem(tree)
            item.setText(0, name)
            for ch,explain in values:
                item_c = QTreeWidgetItem(item)
                item_c.setText(0, ch)
                item_c.setText(1, explain)

    def itemDoubleClick_event(self,item:QTreeWidgetItem,column:int):
        text = item.text(column)

        if text[0] == "Q":
            print("父节点:",text)
            if text == "QPushButton":
                if not self.is_Controls:

                    # 设置
                    self.is_Controls = True
                    self.cu_fun = "randomStyle"
                    self.cu_click_name = "QPushButton"

                    for i in range(self.control_number):
                        btn = QPushButton()
                        btn.setText("test_{}".format(i))
                        btn.setStyleSheet(ButtonStyle.randomStyle())
                        # btn.clicked.connect(partial(self.qss_event,btn.styleSheet(),btn))
                        btn.clicked.connect(partial(self.qss_event,None,btn))
                        self.controls.append(btn)
                        self.addControls(btn)
                else:
                    self.is_Controls = False
                    for w in self.controls:
                        self.flow.removeWidget(w)
                    self.controls.clear()

        else:
            print("子节点:",text)
            if not self.controls:
                return

            # 设置
            self.cu_fun = text

            for w in self.controls:
                if self.cu_click_name == "QPushButton":
                    if text == "randomStyle":
                        w.setStyleSheet(ButtonStyle.randomStyle())
                    elif text == "contrastStyle":
                        w.setStyleSheet(ButtonStyle.contrastStyle())
                    elif text == "homologyStyle":
                        w.setStyleSheet(ButtonStyle.homologyStyle())

    # 控件数量改变事件
    def number_event(self,n:int):
        if not self.controls:
            self.control_number = n
            return

        if n > self.control_number:
            if self.cu_click_name == "QPushButton":
                btn = QPushButton()
                btn.setText("hello")
                self.controls.append(btn)

                if self.cu_fun == "randomStyle":
                    btn.setStyleSheet(ButtonStyle.randomStyle())
                elif self.cu_fun == "contrastStyle":
                    btn.setStyleSheet(ButtonStyle.contrastStyle())
                elif self.cu_fun == "homologyStyle":
                    btn.setStyleSheet(ButtonStyle.homologyStyle())

                # btn.clicked.connect(partial(self.qss_event,btn.styleSheet(),btn))
                btn.clicked.connect(partial(self.qss_event,None,btn))
                self.addControls(btn)
        else:
            pop = self.controls.pop(-1)
            pop.clicked.disconnect()
            self.flow.removeWidget(pop)
        self.control_number = n

    # qss代码事件
    def qss_event(self,qss:str=None,conOBJ:QWidget=None):
        '''
            这里 self.cu_click_control_obj 的清空和赋值的顺序很重要,
            如果先不清空,就赋值,会导致信号额外的触发
        '''
        self.cu_click_control_obj = None

        # 加 try 防止对象不存在报错
        try:
            self.qssEditObj().clear()
            self.qssEditObj().setText(conOBJ.styleSheet())
        except Exception as e:
            print("e:",e)

        # 保存当前点击对象
        self.cu_click_control_obj = conOBJ

    # 修改背景颜色事件
    def background_color_event(self):
        rgb_obj = QColorDialog.getColor()
        rgb = rgb_obj.getRgb()
        self.back_c_btn.setStyleSheet("background-color:rgb({},{},{},{})".format(*rgb))
        self.showObj().setStyleSheet("background-color:rgb({},{},{},{})".format(*rgb))

    # 反射样式
    def reflect_qss_event(self):
        if not self.getClickObj():
            return

        # 加 try 防止对象不存在报错
        try:
            self.getClickObj().setStyleSheet(self.qssEditObj().toPlainText())
        except Exception as e:
            print(e)

    def myEvent(self):
        # 双击树事件
        self.treeObj().itemDoubleClicked.connect(self.itemDoubleClick_event)

        # 控件改变事件
        self.c_number_spin.valueChanged.connect(self.number_event)

        self.back_c_btn.clicked.connect(self.background_color_event)

        # qss代码区文本改变事件
        self.qssEditObj().textChanged.connect(self.reflect_qss_event)

    def resizeEvent(self, e:QResizeEvent) -> None:
        self.updateSize()
        super().resizeEvent(e)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = BuiltStyleDesigner()
    win.show()

    if PYQT_VERSIONS in ["PyQt6", "PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


from PyQtGuiLib.header.versions import PYQT_VERSIONS
if PYQT_VERSIONS == "PyQt5":
    from PyQt5.QtWidgets import (
        QApplication,
        QWidget,
        QFrame,
        QStackedWidget,
        QLabel,
        QPushButton,
        QLineEdit,
        QTextBrowser,
        QGroupBox,
        QGridLayout,
        QVBoxLayout,
        QHBoxLayout,
        QCheckBox,
        QInputDialog,
        QComboBox,
        QMenu,
        QTableWidget,
        QScrollArea,
        QWidgetItem,
        QSplitter,
        QGraphicsOpacityEffect,
        QMainWindow,
        QDesktopWidget as DesktopWidget,
        QGraphicsDropShadowEffect,
        QSlider,
        QListWidget,
        QListWidgetItem,
        QListView,
        QAction,
        QAbstractScrollArea,
        QRadioButton,
        QSpacerItem,
        QSizePolicy,
        QStyle,
        QStyleOption,
        QStyledItemDelegate,
        QLayout,
        QLayoutItem,
        QStyleOptionViewItem,
        QTreeWidget,
        QTreeWidgetItem,
        QTreeWidgetItemIterator,
        QTextBrowser,
        QTextEdit,
        QTabWidget,
        QSpinBox,
        QDoubleSpinBox,
        QFormLayout,
        QColorDialog
    )
if PYQT_VERSIONS == "PyQt6":
    from PyQt6.QtWidgets import (
        QApplication,
        QWidget,
        QFrame,
        QStackedWidget,
        QLabel,
        QPushButton,
        QLineEdit,
        QTextBrowser,
        QGroupBox,
        QGridLayout,
        QVBoxLayout,
        QHBoxLayout,
        QCheckBox,
        QInputDialog,
        QComboBox,
        QMenu,
        QTableWidget,
        QScrollArea,
        QWidgetItem,
        QSplitter,
        QGraphicsOpacityEffect,
        QMainWindow,
        QGraphicsDropShadowEffect,
        QSlider,
        QListWidget,
        QListWidgetItem,
        QListView,
        QAbstractScrollArea,
        QRadioButton,
        QSpacerItem,
        QSizePolicy,
        QStyle,
        QStyleOption,
        QStyledItemDelegate,
        QLayout,
        QLayoutItem,
        QStyleOptionViewItem,
        QTreeWidget,
        QTreeWidgetItem,
        QTreeWidgetItemIterator,
        QTextBrowser,
        QTextEdit,
        QTabWidget,
        QSpinBox,
        QDoubleSpinBox,
        QFormLayout,
        QColorDialog
)

if PYQT_VERSIONS == "PySide2":
    from PySide2.QtWidgets import (
        QApplication,
        QWidget,
        QFrame,
        QStackedWidget,
        QLabel,
        QPushButton,
        QLineEdit,
        QTextBrowser,
        QGroupBox,
        QGridLayout,
        QVBoxLayout,
        QHBoxLayout,
        QCheckBox,
        QInputDialog,
        QComboBox,
        QMenu,
        QTableWidget,
        QScrollArea,
        QWidgetItem,
        QSplitter,
        QGraphicsOpacityEffect,
        QMainWindow,
        QDesktopWidget as DesktopWidget,
        QGraphicsDropShadowEffect,
        QSlider,
        QListWidget,
        QListWidgetItem,
        QListView,
        QAction,
        QAbstractScrollArea,
        QRadioButton,
        QSpacerItem,
        QSizePolicy,
        QStyle,
        QStyleOption,
        QStyledItemDelegate,
        QLayout,
        QLayoutItem,
        QStyleOptionViewItem,
        QTreeWidget,
        QTreeWidgetItem,
        QTreeWidgetItemIterator,
        QTextBrowser,
        QTextEdit,
        QTabWidget,
        QSpinBox,
        QDoubleSpinBox,
        QFormLayout,
        QColorDialog
    )

if PYQT_VERSIONS == "PySide6":
    from PySide6.QtWidgets import (
        QApplication,
        QWidget,
        QFrame,
        QStackedWidget,
        QLabel,
        QPushButton,
        QLineEdit,
        QTextBrowser,
        QGroupBox,
        QGridLayout,
        QVBoxLayout,
        QHBoxLayout,
        QCheckBox,
        QInputDialog,
        QComboBox,
        QMenu,
        QTableWidget,
        QScrollArea,
        QWidgetItem,
        QSplitter,
        QGraphicsOpacityEffect,
        QMainWindow,
        QGraphicsDropShadowEffect,
        QSlider,
        QListWidget,
        QListWidgetItem,
        QListView,
        QAbstractScrollArea,
        QRadioButton,
        QSpacerItem,
        QSizePolicy,
        QStyle,
        QStyleOption,
        QStyledItemDelegate,
        QLayout,
        QLayoutItem,
        QStyleOptionViewItem,
        QTreeWidget,
        QTreeWidgetItem,
        QTreeWidgetItemIterator,
        QTextBrowser,
        QTextEdit,
        QTabWidget,
        QSpinBox,
        QDoubleSpinBox,
        QFormLayout,
        QColorDialog
    )


# -*- coding:utf-8 -*-
# @time:2023/1/10 12:41
# @author:LX
# @file:SlideShow.py
# @software:PyCharm

from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    QApplication,
    sys,
    QWidget,
    QResizeEvent,
    QPushButton,
    QLabel,
    qt,
    QPoint,
    QSize,
    QPropertyAnimation,
    Signal
)

from PyQtGuiLib.styles import ButtonStyle

'''

    轮播图功能
        风格: 扁平
'''

# 滚轮类
class Idler:
    def __init__(self):
        # self.lists = [1,2,3,4,5]
        self.lists = []
        # 双指针
        self.s_p = 0
        self.e_p = 1

        # 首次调用
        self.is_rigth = True
        self.is_left = True

        # 当前点击的方向
        self.cu_d = None

    # 添加窗口
    def addWidget(self,widget:QWidget):
        self.lists.append(widget)

    # 移除窗口
    def removeWidget(self,widget:QWidget):
        self.lists.remove(widget)

    # 长度
    def count(self)->int:
        return len(self.lists)

    # 返回当前两个指针,指向的窗口
    def getWidgtes(self)->tuple:
        if self.cu_d == "left":
            return (self.lists[self.e_p],self.lists[self.s_p])
        return (self.lists[self.s_p],self.lists[self.e_p])

    def widgets(self)->list:
        return self.lists

    # 滚轮向下走一步
    def next(self):
        self.cu_d = "right"
        # 如果首次点击的是下一步
        if self.is_rigth:
            self.s_p,self.e_p = self.count() - 1,0
            self.is_rigth,self.is_left = False,False

        self.s_p += 1
        self.e_p += 1
        if self.s_p > self.count()-1:
            self.s_p = 0
        if self.e_p > self.count()-1:
            self.e_p = 0

    # 滚轮向上走一步
    def up(self):
        self.cu_d = "left"
        self.s_p -= 1
        self.e_p -= 1
        if self.s_p < 0:
            self.s_p = self.count()-1
        if self.e_p < 0:
            self.e_p = self.count()-1



# dl = Idler()
# dl.next()
# print(dl.getWidgtes())
# dl.next()
# print(dl.getWidgtes())
# dl.next()
# print(dl.getWidgtes())
# dl.next()
# print(dl.getWidgtes())
# dl.next()
# print(dl.getWidgtes())
# print("===")
# dl.up()
# print(dl.getWidgtes())
# dl.up()
# print(dl.getWidgtes())



class SlideShow(QWidget):
    # 切换窗口事件
    switchWidgeted = Signal(int)

    # 扁平 风格
    FlatMode = "flat"  # 经典

    # 动画的方向模式
    Ani_Left = "left"
    Ani_Right = "right"
    Ani_Down = "down"
    Ani_Up = "up"

    # 动画
    Translation = "translation"  # 平移

    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(700, 300)

        self.show_mode = SlideShow.FlatMode # 展示模式
        self.animation_mode = SlideShow.Translation  # 动画模式

        # 滚轮类
        self.idler = Idler()

        self.animation_time = 300 # 毫秒

        # 设置自动轮播,自动轮播的方向组
        self.is_auto_slide = False
        self.auto_direction =SlideShow.Ani_Right

        # 当前显示的窗口
        self.current_widget = None  # type:QWidget

        self.createBtn()

        for i in range(1,5):
            l = QLabel(str(i))
            l.setAlignment(qt.AlignCenter)
            l.setStyleSheet(ButtonStyle.contrastStyle())
            self.addWidget(l)

    # 创建按钮
    def createBtn(self):
        # 左右按钮
        self.Ani_Left_btn = QPushButton("左", self)
        self.Ani_Right_btn = QPushButton("右", self)
        self.Ani_Left_btn.setObjectName("Ani_Left_btn")
        self.Ani_Right_btn.setObjectName("Ani_Right_btn")
        self.Ani_Left_btn.resize(50, 50)
        self.Ani_Right_btn.resize(50, 50)
        self.btnStyle()
        self.Ani_Left_btn.clicked.connect(lambda :self.LR_event(SlideShow.Ani_Left))
        self.Ani_Right_btn.clicked.connect(lambda :self.LR_event(SlideShow.Ani_Right))

    # 按钮样式
    def btnStyle(self):
        self.Ani_Left_btn.setStyleSheet('''
#Ani_Left_btn{
background-color:transparent;
border:2px solid #73ffcc;
color:#73ffcc;
border-radius:5px;
}
#Ani_Left_btn:hover{
border:2px solid #00557f;
}
        ''')
        self.Ani_Right_btn.setStyleSheet('''
#Ani_Right_btn{
background-color:transparent;
border:2px solid #73ffcc;
color:#73ffcc;
border-radius:5px;
}
#Ani_Right_btn:hover{
border:2px solid #00557f;
}
                ''')

    # 两侧的按钮
    def btnPos(self):
        self.Ani_Left_btn.move(5, self.height() // 2 - self.Ani_Left_btn.height() // 2)
        self.Ani_Right_btn.move(self.width() - self.Ani_Right_btn.width() - 5,
                            self.height() // 2 - self.Ani_Left_btn.height() // 2)

    # 按钮事件
    def LR_event(self,d:str):
        self.idler.next()
        self.__animation()

    # 添加窗口
    def addWidget(self,widget:QWidget):
        widget.setParent(self)
        widget.lower()
        widget.resize(self.size())

        self.current_widget = widget
        self.hideItWidget()
        self.idler.addWidget(widget)

    # 隐藏其他窗口
    def hideItWidget(self):
        if not self.current_widget:
            return
        for w in self.idler.widgets():
            if self.current_widget != w and not w.isHidden():
                w.hide()

    # 移除窗口
    def removeWidget(self,widget:QWidget):
        self.idler.removeWidget(widget)

    # 切换到指定窗口(暂时放弃改函数)
    def setCurrentIndex(self,index:int=0,is_animation:bool=False):
        pass

    # 过渡动画
    def __animation(self):
        if self.auto_direction == SlideShow.Ani_Up:
            pass
        elif self.auto_direction == SlideShow.Ani_Down:
            pass
        elif self.auto_direction == SlideShow.Ani_Left:
            pass
        else: # 默认右
            l = self.idler.getWidgtes()[0]
            r = self.idler.getWidgtes()[1]
            print("当前:",l.text(),r.text())
            # self.auto_direction == SlideShow.Ani_Right:
            # pass


    def resizeEvent(self, event:QResizeEvent) -> None:
        self.btnPos()
        # 只渲染当前的页面
        if self.current_widget:
            self.current_widget.resize(event.size())

        super().resizeEvent(event)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = SlideShow()
    win.show()

    if PYQT_VERSIONS in ["PyQt6", "PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())




from PyQtGuiLib.header import (
    sys,
    QApplication,
    QPropertyAnimation,
    QWidget,
    QPushButton,
    PYQT_VERSIONS,
    QGraphicsOpacityEffect,
    QThread,
    Signal
)


class ZoomAnimation(QPropertyAnimation):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)


from PyQtGuiLib.header import (
    QWidget,
    Signal,
    Qt,
    QFont,
    QColor,
    QPen,
    QPainter,
    QPaintEvent,
    QSize,
    QResizeEvent,
    textSize,
    qt
)



'''
    圆环进度条
'''
class CircularBar(QWidget):
    # 进度改变时,发出信号
    valueChange = Signal(int)

    # 外圈/内圈
    OuterRing = "OuterRing"
    InnerRing = "InnerRing"
    Double = "Double"

    # 圈的风格
    SolidLine = qt.SolidLine
    DashLine = qt.DashLine
    DotLine = qt.DotLine
    DashDotLine = qt.DashDotLine
    DashDotDotLine = qt.DashDotDotLine
    CustomDashLine = qt.CustomDashLine


    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)

        self.w,self.h = 150,150
        self.degree = 3.6  # 每一段的大小
        self.outer_n = 0 # 总量
        self.inner_n = 0

        # 文字
        self.text = "0%"
        self.text_color = QColor(104, 167, 205)
        self.text_size = 25

        '''
            进度发生变化时,引起变化的线段,是外圈还是内圈
        '''
        self.variableLine = CircularBar.InnerRing

        self.__max_value = 100

        # 外圈/内圈 颜色
        self.outer_rgb = QColor(104, 167, 205)
        self.inner_rgb = QColor(117, 199, 255)

        # 外圈/内圈 线段样式
        self.outer_style = CircularBar.CustomDashLine
        self.inner_style = CircularBar.SolidLine

        # 外圈/内圈 线段长度
        self.outer_width = 10
        self.inner_width = 3

        # 外圈/内圈 自定义风格  只有当线段样式为 CircularBar.CustomDashLine 才有效
        self.outer_pattern = []
        self.inner_pattern = []

    def resize(self, *args) -> None:
        if len(args) == 1 and isinstance(args[0],QSize):
            self.w = args[0].width(),args[0].height()
        else:
            self.w,self.h = args[0],args[1]

        super().resize(self.w+10,self.h+10)

    def setText(self,text:str):
        self.text = text

    def setTextColor(self,color:QColor):
        self.text_color = color

    def setTextSize(self,size:int):
        self.text_size = size

    def setAllText(self,text:str,color:QColor,size:int):
        self.setText(text)
        self.setTextColor(color)
        self.setTextSize(size)

    # 设置外圈颜色
    def setOuterColor(self,color:QColor):
        self.outer_rgb = color

    # 设置内圈颜色
    def setInnerColor(self,color:QColor):
        self.inner_rgb = color

    # 设置外圈风格
    def setOuterStyle(self,style):
        self.outer_style = style

    # 设置内圈风格
    def setInnerStyle(self,style):
        self.inner_style = style

    # 设置变化的线段
    def setVariableLineSegment(self,mode):
        self.variableLine = mode

    def setValue(self,v:int):
        if v > self.__max_value:
            raise ValueError("The maximum is 100 !")

        if self.variableLine == CircularBar.OuterRing:
            self.outer_n = int(self.degree*v)
            self.inner_n = 360
        elif self.variableLine == CircularBar.InnerRing:
            self.outer_n = 360
            self.inner_n = int(self.degree*v)
        elif self.variableLine == CircularBar.Double:
            self.outer_n = int(self.degree * v)
            self.inner_n = int(self.degree * v)

        self.text = "{}%".format(v)
        self.update()
        # 发射信号
        self.valueChange.emit(v)

    def value(self) -> int:
        return int(self.n/self.degree)

    def maxValue(self) -> int:
        return self.__max_value

    # 设置外圈自定义线段样式
    def setOuterDashPattern(self,pattern:list):
        self.outer_pattern = pattern

    # 设置内圈自定义线段样式
    def setInnerDashPattern(self,pattern:list):
        self.inner_pattern = pattern

    # 画环
    def drawLoopBar(self,painter:QPainter,x,y):
        inner_x,inner_y = x+10,y+10
        inner_w,inner_h = self.w-20,self.h-20

        # ------------
        outer_op = QPen(self.outer_rgb,self.outer_width)
        outer_op.setStyle(self.outer_style)
        if self.outer_style == CircularBar.CustomDashLine:
            outer_op.setDashPattern(self.outer_pattern)
        painter.setPen(outer_op)
        painter.drawArc(x,y,self.w,self.h,0,self.outer_n*16)
        inner_op = QPen(self.inner_rgb, self.inner_width)
        inner_op.setStyle(self.inner_style)
        if self.inner_style == CircularBar.CustomDashLine:
            inner_op.setDashPattern(self.inner_pattern)
        painter.setPen(inner_op)
        painter.drawArc(inner_x,inner_y,inner_w,inner_h, 0, self.inner_n * 16)
        # ---------------
        # 绘制文字
        f = QFont()
        f.setPointSize(self.text_size)
        painter.setFont(f)
        painter.setPen(self.text_color)
        # 文字
        fs = textSize(f,self.text)
        fw = fs.width()
        fh = fs.height()
        painter.drawText(inner_x+inner_w//2-fw//2,y+self.h//2+fh//4,self.text)

    def paintEvent(self, event:QPaintEvent) -> None:
        painter = QPainter()
        painter.begin(self)
        painter.setRenderHints(qt.Antialiasing | qt.SmoothPixmapTransform | qt.TextAntialiasing)

        self.drawLoopBar(painter,6,6)

        painter.end()

    # 跟随窗口变化
    def resizeEvent(self, event:QResizeEvent) -> None:
        self.w,self.h = event.size().width()-13,event.size().height()-13
        super().resizeEvent(event)



from ctypes import POINTER, Structure
from ctypes.wintypes import DWORD, ULONG
from enum import Enum


class WINDOWCOMPOSITIONATTRIB(Enum):
    WCA_UNDEFINED = 0,
    WCA_NCRENDERING_ENABLED = 1,
    WCA_NCRENDERING_POLICY = 2,
    WCA_TRANSITIONS_FORCEDISABLED = 3,
    WCA_ALLOW_NCPAINT = 4,
    WCA_CAPTION_BUTTON_BOUNDS = 5,
    WCA_NONCLIENT_RTL_LAYOUT = 6,
    WCA_FORCE_ICONIC_REPRESENTATION = 7,
    WCA_EXTENDED_FRAME_BOUNDS = 8,
    WCA_HAS_ICONIC_BITMAP = 9,
    WCA_THEME_ATTRIBUTES = 10,
    WCA_NCRENDERING_EXILED = 11,
    WCA_NCADORNMENTINFO = 12,
    WCA_EXCLUDED_FROM_LIVEPREVIEW = 13,
    WCA_VIDEO_OVERLAY_ACTIVE = 14,
    WCA_FORCE_ACTIVEWINDOW_APPEARANCE = 15,
    WCA_DISALLOW_PEEK = 16,
    WCA_CLOAK = 17,
    WCA_CLOAKED = 18,
    WCA_ACCENT_POLICY = 19,
    WCA_FREEZE_REPRESENTATION = 20,
    WCA_EVER_UNCLOAKED = 21,
    WCA_VISUAL_OWNER = 22,
    WCA_LAST = 23


class ACCENT_STATE(Enum):
    ACCENT_DISABLED = 0,
    ACCENT_ENABLE_GRADIENT = 1,
    ACCENT_ENABLE_TRANSPARENTGRADIENT = 2
    ACCENT_ENABLE_BLURBEHIND = 3,
    ACCENT_ENABLE_ACRYLICBLURBEHIND = 4,
    ACCENT_INVALID_STATE = 5


class ACCENT_POLICY(Structure):
    _fields_ = [
        ('AccentState',   DWORD),
        ('AccentFlags',   DWORD),
        ('GradientColor', DWORD),
        ('AnimationId',   DWORD),
    ]


class WINDOWCOMPOSITIONATTRIBDATA(Structure):
    _fields_ = [
        ('Attribute',  DWORD),
        ('Data',       POINTER(ACCENT_POLICY)),
        ('SizeOfData', ULONG),
    ]


import platform
from PyQtGuiLib.header import PYQT_VERSIONS,DesktopWidget,QPoint,QSize,QFontMetricsF,QFont

is_win_sys = True if platform.system() == "win32" else False

is_mac_sys = True if platform.system() == "Darwin" else False


'''
    这几个与屏幕有关的方法,只能在窗口中调用,否则报错
'''

# 获取单个桌面大小
def desktopSize() -> QSize:
    if PYQT_VERSIONS == "PyQt5":
        from PyQt5.QtWidgets import QApplication
        size = QApplication.desktop().size()
        count = QApplication.desktop().screenCount()
        return QSize(size.width()//count,size.height())
    elif PYQT_VERSIONS in ["PyQt6","PySide2","PySide6"]:
        return DesktopWidget.primaryScreen().availableGeometry().size()
    else:
        return QSize(0,0)


# 桌面居中位置
def desktopCenter() -> QPoint:
    if PYQT_VERSIONS == "PyQt5":
        return DesktopWidget().availableGeometry().center()
    elif PYQT_VERSIONS in ["PyQt6","PySide2","PySide6"]:
        return DesktopWidget.primaryScreen().availableGeometry().center()
    else:
        return QPoint(0,0)


# 获取文字大小
def textSize(font:QFont,text:str)->QSize:
    fs = QFontMetricsF(font)
    if PYQT_VERSIONS == "PyQt5":
        return QSize(int(fs.width(text)),int(fs.height()))
    elif PYQT_VERSIONS in ["PyQt6","PySide2","PySide6"]:
        return QSize(int(fs.horizontalAdvance(text)+1), int(fs.height()+1)) # +1 是为了补偿丢失的像素
    else:
        return QSize(0,0)


from PyQtGuiLib.header import QWidget


# 控件的组成分析函数
def dumpStructure(widget :QWidget ,spaceCount=0):
    '''
        widget:控件
        spaceCount:边距
    '''
    for _ in range(spaceCount):
        print("-" ,end="")
    print("\033[34;5m[{}\033[0m".format(widget.metaObject().className()),end="")
    wObj = widget.objectName()
    if wObj:
        print(":{}".format(widget.objectName()),end="")
    else:
        print(":[]", end="")
    print("\033[34;5m]\033[0m")
    # print("[{}:{}]".format(widget.metaObject().className(), widget.objectName()))

    for ch in widget.children():
        dumpStructure(ch,spaceCount + 4)